////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	AVR32_Device_JTAG.c
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	JTAG Programming for AVR32
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>

#include "../../type.h"
#include "../../app_cfg.h"

#include "../../MCU/gpio.h"

#include "../../Delay/delay.h"

#include "../AVR_Device_Const.h"
#include "../AVR_Device_Common.h"

#include "AVR32_Device_JTAG.h"

#include "../../AVR_Programming/JTAG/JTAG_TAP.h"
#include "../../AVR_Programming/JTAG/AVR32_JTAG.h"

#if AVR32P_JTAG_SHOULD_ENABLE
/// JTAG Programming for AVR32
/// @param[in]	dat		Command Array
/// @param[in]	len		Command Length
/// @return		Command Result
uint8 AVR32P_JTAG_ProcessProgCmd(uint8* dat,uint16 len)
{
	switch(dat[0])
	{
	case JTAGICE_CMND_GET_PARAMETER:
		switch(dat[1])
		{
		case PARAM_JTAGID:
			break;
		default:
			_RSP_ILLEGAL_PARAMETER();
			break;
		}
	case JTAGICE_CMND_WRITE_MEMORY:
		switch(dat[1])
		{
		case MEMORY_TYPE_FLASH_PAGE:
			break;
		case MEMORY_TYPE_EEPROM_PAGE:
			break;
		case MEMORY_TYPE_FUSE_BITS:
			break;
		case MEMORY_TYPE_LOCK_BITS:
			break;
		default:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		}
		break;
	case JTAGICE_CMND_READ_MEMORY:
		switch(dat[1])
		{
		case MEMORY_TYPE_FLASH_PAGE:
			break;
		case MEMORY_TYPE_EEPROM_PAGE:
			break;
		case MEMORY_TYPE_FUSE_BITS:
			break;
		case MEMORY_TYPE_LOCK_BITS:
			break;
		case MEMORY_TYPE_SIGN_JTAG:
			break;
		case MEMORY_TYPE_OSCCAL_BYTE:
			break;
		default:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		}
		break;
	case JTAGICE_CMND_CHIP_ERASE:
		break;
	case JTAGICE_CMND_ENTER_PROGMODE:
		break;
	case JTAGICE_CMND_LEAVE_PROGMODE:
		break;
	}

	return 0;
}
#endif		// #if AVR32P_JTAG_SHOULD_ENABLE
