////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	AVR_Device_Common.h
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	headerfile for Atmel Tools
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////

#define RSP_MAX_PKGSIZE				278
#define CMD_MAX_PKGSIZE				317

extern uint8 Send_Sign_On;

#if (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRISP_MKII)
extern uint8 Vtarget;
#define Vtarget_HasPower			27
#elif (AVR_DEVICE_ATMEL == AVR_DEVICE_JTAGICE_MKII) || (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRDRAGON)
extern uint16 Vtarget;
#define Vtarget_HasPower			2700
#endif

#if AVRP_HVPP_SHOULD_ENABLE || AVRP_HVSP_SHOULD_ENABLE
extern uint8 hvControlStack[32];
extern const uint8 HV_PageSize[8];
#endif

#if AVRP_ISP_SHOULD_ENABLE && ISP_AutoSpeed_En
uint8 ISP_AutoSpeed;
#endif

#if AVRP_ISP_SHOULD_ENABLE
extern uint8 Program_Speed;
extern uint8 Reset_Polarity;
extern uint8 Discharge_Delay;
#endif
#if (AVR_DEVICE_ATMEL == AVR_DEVICE_JTAGICE_MKII) || (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRDRAGON)
uint8 Emulator_Mode;
#endif
#if AVRP_JTAG_SHOULD_ENABLE || AVRXP_JTAG_SHOULD_ENABLE || AVR32P_JTAG_SHOULD_ENABLE
uint32 JTAGID_String;
//uint8 JTAG_Clk;
#endif
#if AVRD_JTAG_SHOULD_ENABLE || AVRD_DW_SHOULD_ENABLE
uint8 OCD_Break_Cause;
//uint8 Timers_Running;
//uint8 Break_on_Charge_of_Flow;
uint8 Target_MCU_State = 0;
//uint8 Daisy_Chain_Info[4];
//uint8 Boot_Address[4];
uint32 Program_Entry_Point;
//uint8 CAN_Flag;
uint8 TargetPower = 0;
#endif

// Part Descriptions
#if AVRP_JTAG_SHOULD_ENABLE
#if AVRD_JTAG_SHOULD_ENABLE
// Used in JTAGICE_CMND_SET_DEVICE_DESCRIPTOR
extern uint8 ucReadIO[8];
extern uint8 ucReadIOShadow[8];
extern uint8 ucWriteIO[8];
extern uint8 ucWriteIOShadow[8];
extern uint8 ucReadExtIO[52];
extern uint8 ucReadIOExtShadow[52];
extern uint8 ucWriteExtIO[52];
extern uint8 ucWriteIOExtShadow[52];
extern uint8 ucIDRAddress;
extern uint8 ucSPMCRAddress;
extern uint32 ulBootAddress;
extern uint8 ucRAMPZAddress;
extern uint16 uiUpperExtIOLoc;
extern uint32 ulFlashSize;
extern uint8 ucEepromInst[20];
extern uint8 ucFlashInst[3];
extern uint8 ucSPHaddr;
extern uint8 ucSPLaddr;
extern uint16 uiFlashpages;
extern uint8 ucDWDRAddress;
extern uint8 ucDWBasePC;
extern uint16 uiStartSmallestBootLoaderSection;
extern uint8 ucEnablePageProgramming;
extern uint8 ucCacheType;
extern uint16 uiSramStartAddr;
extern uint8 ucResetType;
extern uint8 ucPCMaskExtended;
extern uint8 ucPCMaskHigh;
extern uint8 ucEindAddress;
extern uint16 uiEECRAddress;
#endif
extern uint8 uiFlashPageSize;			// Device Flash Page Size in word
extern uint8 ucEepromPageSize;			// Device Eeprom Page Size in bytes
extern uint8 ucAllowFullPageBitstream;
#endif

extern uint8 AVRP_Programming;

#if (AVR_DEVICE_ATMEL == AVR_DEVICE_JTAGICE_MKII) || (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRDRAGON)
#define CRC_INIT				0xFFFF
extern uint16 CRC_Val;
void CRC16(uint8);
#endif

#if AVR_DEVICE_ATMEL == AVR_DEVICE_AVRISP_MKII
uint8 AVR_Device_Name[10];
#elif AVR_DEVICE_ATMEL == AVR_DEVICE_JTAGICE_MKII
uint8 AVR_Device_Name[12];
#elif AVR_DEVICE_ATMEL == AVR_DEVICE_AVRDRAGON
uint8 AVR_Device_Name[9];
#elif AVR_DEVICE == AVR_DEVICE_STK600
uint8 AVR_Device_Name[6];
#endif


extern uint8 AVR_Device_DataBuff[CMD_MAX_PKGSIZE];

extern uint16 RSP_Len;
extern uint16 RSP_Len_tmp;

extern uint16 CMD_Len;
extern uint16 CMD_Len_tmp;

#if (AVR_DEVICE_ATMEL == AVR_DEVICE_JTAGICE_MKII) || (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRDRAGON)
extern const uint16 crc_tbl[256];
#endif

#if AVRP_ISP_SHOULD_ENABLE || AVRP_HVPP_SHOULD_ENABLE || AVRP_HVSP_SHOULD_ENABLE
extern uint8 __prog_addr_over_64k;
extern uint16 __prog_addr;
#endif

void AVRP_ProcessCmd(uint8*,uint16);
#if (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRISP_MKII)
#define AVRP_TxLen	RSP_Len
#define AVR_Device_ProcessCmd AVRP_ProcessCmd
#else
extern uint16 AVRP_TxLen;
void AVR_Device_ProcessCmd(uint8*,uint16);
#endif

#if AVRD_JTAG_SHOULD_ENABLE || AVRD_DW_SHOULD_ENABLE
void AVR_Device_CheckEvent(void);
#endif

#if (AVR_DEVICE_ATMEL == AVR_DEVICE_AVRISP_MKII)
void AVR_Device_Upgrade(void);
#endif
