////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	AVR_Device_JTAG.c
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	JTAG Programming and Debugging for AVR under STK500v2 Protocol
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>

#include "../../type.h"
#include "../../app_cfg.h"

#include "../../MCU/gpio.h"

#include "../../Delay/delay.h"

#include "../AVR_Device_Const.h"
#include "../AVR_Device_Common.h"

#include "AVR_Device_JTAG.h"

#include "../../AVR_Programming/JTAG/JTAG_TAP.h"
#include "../../AVR_Programming/JTAG/AVR_JTAG.h"

#if AVRD_JTAG_SHOULD_ENABLE
#include "../../AVR_Debugging/AVR_Instr.h"
#include "../../AVR_Debugging/JTAG/AVR_JTAGOCD.h"
#endif

#if AVRD_DW_SHOULD_ENABLE
#include "../../AVR_Debugging/DW/AVR_DWOCD.h"
#endif

#if AVRP_JTAG_SHOULD_ENABLE
/// JTAG Programming and Debugging for AVR under STK500v2 Protocol
/// @param[in]	dat		Command Array
/// @param[in]	len		Command Length
/// @return		Command Result
uint8 AVRP_JTAG_ProcessProgCmd(uint8* dat,uint16 len)
{
#if AVRD_JTAG_SHOULD_ENABLE || AVRD_DW_SHOULD_ENABLE
	uint16 startaddr;
#endif
#if AVRP_JTAG_SHOULD_ENABLE
	uint16 i,length;
	uint8 highbyte,lowbyte,fuse_tmp[3],tmp8;
#endif

	switch(dat[0])
	{
	case JTAGICE_CMND_SET_PARAMETER:
#if AVRP_JTAG_SHOULD_ENABLE
		switch(dat[1])
		{
#if AVRD_JTAG_SHOULD_ENABLE
		case 0x29:
		{
uint8 tdi[4],tdo[4];
			if(!AVRP_Programming)
			{
				AVRP_Programming = 2;
				JTAG_TAP_Init();
				JTAG_TAP_Reset();
			}

			AVR_JTAGOCD_Init();

			// Enable PSB0 for ProgramEntryPoint Break
			AVR_JTAGOCD_WriteOCD(AVR_JTAG_OCDREG_PSB0,(uint8*)&Program_Entry_Point);
			AVR_JTAGOCD_ReadOCD(AVR_JTAG_OCDREG_BCR,tdo);
			tdi[0] = 0;
			tdi[1] = AVR_JTAG_BCR_EN_PSB0;
			AVR_JTAGOCD_WriteOCD(AVR_JTAG_OCDREG_BCR,tdi);
#if 0
// For Test

// Release Reset
AVR_JTAG_SendIns(JTAG_INS_AVR_RESET);
JTAG_TAP_Data_1Byte(0,1,JTAG_TAP_ENTER_IDLE);

// Wait to break on ProgrmEntryPoint
DelayMS(200);

//AVR_JTAGOCD_ReadOCD(AVR_JTAG_OCDREG_BSR,tdi);

//AVR_JTAGOCD_ReadOCD(AVR_JTAG_OCDREG_COMM_CTL,tdo);
//AVR_JTAGOCD_ReadOCD(AVR_JTAG_OCDREG_BCR,tdo);
//tdi[0] = tdi[1] = 0;
//AVR_JTAGOCD_WriteOCD(AVR_JTAG_OCDREG_BCR,tdi);

//AVR_JTAGOCD_CheckPC(tdo);
//AVRI_RJMP_k(-6);
AVRI_RunInstr(0x02,0xC0);	// 0xC002		RJMP 6
AVR_JTAGOCD_CheckPC(tdo);
//AVR_JTAGOCD_CheckPC(tdo);
//AVR_JTAGOCD_CheckPC(tdo);

// Enable OCDR,change OCDR,read OCDR    OK
//AVR_JTAGOCD_ReadOCD(AVR_JTAG_OCDREG_COMM_CTL,tdo);
//tdo[1] |= AVR_JTAG_CTL_EN_OCDR;
//AVR_JTAGOCD_WriteOCD(AVR_JTAG_OCDREG_COMM_CTL,tdo);
//AVRI_LDI_Rd_K(16,0xA5);			// LDI R16,0xA5
//AVRI_OUT_A_Rr(ucIDRAddress,16);	// OUT ucIDRAddress,R16
//AVR_JTAGOCD_ReadOCDR(tdo);

dat[1] = tdo[0];
dat[2] = tdo[1];
dat[3] = tdo[2];
dat[4] = tdo[3];
dat[5] = tdi[0];
dat[6] = tdi[1];
dat[7] = tdi[2];
dat[8] = tdi[3];
_RSP_MEMORY(8);
return 0;
#endif
		}
			break;
		default:
			//_RSP_ILLEGAL_PARAMETER();
			break;
#endif
		}
#endif
		_RSP_OK();
		break;
	case JTAGICE_CMND_GET_PARAMETER:
		switch(dat[1])		// Parameter ID
		{
#if AVRP_JTAG_SHOULD_ENABLE
		case PARAM_JTAGID:
			if(!AVRP_Programming)
			{
#if ISP_JTAG_AutoPower_En
#if !ISP_JTAG_FourcePower_En
				if(Vtarget < Vtarget_HasPower)
#endif
				{
					ISP_JTAG_PowerOn();
					DelayMS(5);
				}
#endif
				JTAG_TAP_Init();
				JTAG_TAP_Reset();
			}
			AVR_JTAG_SendIns(AVR_JTAG_INS_IDCODE);

			JTAG_TAP_Data(0,(uint8*)&JTAGID_String,AVR_JTAG_REG_JTAGID_Len,JTAG_TAP_ENTER_IDLE);

			if(!AVRP_Programming)
			{
#if ISP_JTAG_AutoPower_En
				ISP_JTAG_PowerOff();
#endif
				JTAG_TAP_Fini();
			}

			PrepareRSP(RSP_PARAMETER,5);
			memcpy(&dat[1],&JTAGID_String,4);
			break;
#endif
		default:
			_RSP_ILLEGAL_PARAMETER();
			break;
		}
		break;
#if AVRP_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_WRITE_MEMORY:
#if AVRD_JTAG_SHOULD_ENABLE
		startaddr = dat[6] + ((uint16)dat[7] << 8);
		length = dat[2] + ((uint16)dat[3] << 8);
#endif
		switch(dat[1])
		{
#if AVRD_JTAG_SHOULD_ENABLE
		case MEMORY_TYPE_IO_SHADOW:
			break;
		case MEMORY_TYPE_SRAM:
			break;
		case MEMORY_TYPE_EEPROM:
			break;
		case MEMORY_TYPE_EVENT:
			break;
		case MEMORY_TYPE_EVENT + 1:
			_RSP_OK();
			break;
		case MEMORY_TYPE_SPM:
			break;
#endif
		case MEMORY_TYPE_FLASH_PAGE:
			dat[6] >>= 1;
			if(dat[7] & 0x01)
				dat[6] |= 0x80;
			dat[7] >>= 1;
			if(dat[8] & 0x01)
				dat[7] |= 0x80;

			highbyte = dat[7];
			lowbyte = dat[6];

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterFlashWrite();
			if(dat[9])
				AVR_JTAG_PROG_LoadAddrExtendedHighByte(1);
			AVR_JTAG_PROG_LoadAddrHighByte(highbyte);

			AVR_JTAG_PROG_LoadAddrLowByte(lowbyte);
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_PAGELOAD);

			if(ucAllowFullPageBitstream)
			{
				JTAG_TAP_DataOut(dat + 10,uiFlashPageSize * 16,JTAG_TAP_ENTER_IDLE);
			}
			else
			{
				for(i = 0;i < (uint16)uiFlashPageSize * 2;i++)
					JTAG_TAP_Data_1Byte(dat[10 + i],8,0);
				JTAG_TAP_Enter_Idle_Mode();
			}

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);

			AVR_JTAG_PROG_WriteFlashPage();
			while(!AVR_JTAG_PROG_WriteFlashPageComplete());

			_RSP_OK();
			break;
		case MEMORY_TYPE_EEPROM_PAGE:
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterEEPROMWrite();

			AVR_JTAG_PROG_LoadAddrHighByte(dat[7]);
			for(i = 0;i < ucEepromPageSize;i++)
			{
				AVR_JTAG_PROG_LoadAddrLowByte(dat[6] + i);
				AVR_JTAG_PROG_LoadDataByte(dat[10 + i]);
				AVR_JTAG_PROG_LatchData();
			}

			AVR_JTAG_PROG_WriteEEPROMPage();
			while(!AVR_JTAG_PROG_WriteEEPROMPageComplete());

			_RSP_OK();
			break;
		case MEMORY_TYPE_FUSE_BITS:
			lowbyte = dat[2];		// number of bytes
			highbyte = dat[6];		// address

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterFuseWrite();

			for(tmp8 = 0;tmp8 < lowbyte;tmp8++)
			{
				if((highbyte + tmp8) == 0)
				{
					AVR_JTAG_PROG_LoadDataLowByte(dat[10 + tmp8]);
					AVR_JTAG_PROG_WriteFuseLowByte();
					//DelayMS(10);
					while(!AVR_JTAG_PROG_WriteFuseExtByteComplete());
				}
				else if((highbyte + tmp8) == 1)
				{
					AVR_JTAG_PROG_LoadDataLowByte(dat[10 + tmp8]);
					AVR_JTAG_PROG_WriteFuseHighByte();
					//DelayMS(10);
					while(!AVR_JTAG_PROG_WriteFuseExtByteComplete());
				}
				else
				{
					AVR_JTAG_PROG_LoadDataLowByte(dat[10 + tmp8]);
					AVR_JTAG_PROG_WriteFuseExtByte();
					//DelayMS(10);
					while(!AVR_JTAG_PROG_WriteFuseExtByteComplete());
				}
			}

			_RSP_OK();
			break;
		case MEMORY_TYPE_LOCK_BITS:
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterLockbitWrite();

			AVR_JTAG_PROG_LoadDataByte(dat[10]);
			AVR_JTAG_PROG_WriteLockbit();
			while(!AVR_JTAG_PROG_WriteLockbitComplete());

			_RSP_OK();
			break;
		default:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		}
		break;
	case JTAGICE_CMND_READ_MEMORY:
#if AVRD_JTAG_SHOULD_ENABLE
		startaddr = dat[6] + ((uint16)dat[7] << 8);
#endif
		length = dat[2] + ((uint16)dat[3] << 8);
		switch(dat[1])
		{
#if AVRD_JTAG_SHOULD_ENABLE
		case MEMORY_TYPE_IO_SHADOW:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		case MEMORY_TYPE_SRAM:
			AVR_JTAGOCD_SaveZR16();
			if(startaddr < 0x20)
				AVR_JTAGOCD_ReadGPR(startaddr,dat + 1,length);				// Read General Purpose Registers
			else if(startaddr < 0x60)
				AVR_JTAGOCD_ReadIO(startaddr - 0x20,dat + 1,length);		// Read IO
			else
				AVR_JTAGOCD_ReadSRAM(startaddr,dat + 1,length);				// Read SRAM
			AVR_JTAGOCD_RestoreZR16();

			_RSP_MEMORY(length);
			break;
		case MEMORY_TYPE_EEPROM:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		case MEMORY_TYPE_EVENT:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		case MEMORY_TYPE_SPM:
			AVR_JTAGOCD_ReadFlash(startaddr,dat + 1,length);

			_RSP_MEMORY(length);
			break;
#endif
		case MEMORY_TYPE_FLASH_PAGE:
			dat[6] >>= 1;
			if(dat[7] & 0x01)
				dat[6] |= 0x80;
			dat[7] >>= 1;
			if(dat[8] & 0x01)
				dat[7] |= 0x80;

			highbyte = dat[7];
			lowbyte = dat[6];

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterFlashRead();
			if(dat[9])
				AVR_JTAG_PROG_LoadAddrExtendedHighByte(1);
			AVR_JTAG_PROG_LoadAddrHighByte(highbyte);

			AVR_JTAG_PROG_LoadAddrLowByte(lowbyte);
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_PAGEREAD);

			if(ucAllowFullPageBitstream)
			{
				JTAG_TAP_Data_1Byte(0,8,JTAG_TAP_ENTER_IDLE);

				JTAG_TAP_DataIn(dat + 1,uiFlashPageSize * 16,JTAG_TAP_ENTER_IDLE);
			}
			else
			{
				for(i = 0;i < (uint16)uiFlashPageSize * 2;i++)
					dat[1 + i] = JTAG_TAP_Data_1Byte(0,8,0);
				JTAG_TAP_Enter_Idle_Mode();
			}

			_RSP_MEMORY((uint16)uiFlashPageSize * 2);
			break;
		case MEMORY_TYPE_EEPROM_PAGE:
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterEEPROMRead();

			highbyte = dat[7];
			lowbyte = dat[6];

			for(i = 0;i < length;i++)
			{
				AVR_JTAG_PROG_LoadAddrHighByte(highbyte);
				AVR_JTAG_PROG_LoadAddrLowByte(lowbyte + i);
				AVR_JTAG_PROG_INS(0x3300 | (lowbyte + i));
				AVR_JTAG_PROG_INS(0x3200);
				dat[1 + i] = AVR_JTAG_PROG_INS(0x3300) & 0xFF;
			}
			_RSP_MEMORY(length);
			break;
		case MEMORY_TYPE_FUSE_BITS:
			lowbyte = dat[2];		// number of bytes
			highbyte = dat[6];		// address

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterFuseLockbitRead();

			fuse_tmp[0] = AVR_JTAG_PROG_ReadFuseLowByte();
			fuse_tmp[1] = AVR_JTAG_PROG_ReadFuseHighByte();
			fuse_tmp[2] = AVR_JTAG_PROG_ReadExtFuseByte();

			for(tmp8 = 0;tmp8 < lowbyte;tmp8++)
				dat[1 + tmp8] = fuse_tmp[tmp8 + highbyte];

			_RSP_MEMORY(lowbyte);
			break;
		case MEMORY_TYPE_LOCK_BITS:
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterFuseLockbitRead();

			dat[1] = AVR_JTAG_PROG_ReadLockbit();

			_RSP_MEMORY(1);
			break;
		case MEMORY_TYPE_SIGN_JTAG:
			lowbyte = dat[2];		// number of bytes
			highbyte = dat[6];		// address

			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterSignByteRead();

			AVR_JTAG_PROG_LoadAddrByte(0);
			fuse_tmp[0] = AVR_JTAG_PROG_ReadSignByte();
			AVR_JTAG_PROG_LoadAddrByte(1);
			fuse_tmp[1] = AVR_JTAG_PROG_ReadSignByte();
			AVR_JTAG_PROG_LoadAddrByte(2);
			fuse_tmp[2] = AVR_JTAG_PROG_ReadSignByte();

			for(tmp8 = 0;tmp8 < lowbyte;tmp8++)
				dat[1 + tmp8] = fuse_tmp[tmp8 + highbyte];

			_RSP_MEMORY(lowbyte);
			break;
		case MEMORY_TYPE_OSCCAL_BYTE:
			AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
			AVR_JTAG_PROG_EnterCaliByteRead();

			AVR_JTAG_PROG_LoadAddrByte(dat[6]);
			dat[1] = AVR_JTAG_PROG_ReadCaliByte();

			_RSP_MEMORY(1);
			break;
		default:
			_RSP_ILLEGAL_MEMORY_TYPE();
			break;
		}
		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_WRITE_PC:
		break;
	case JTAGICE_CMND_READ_PC:
		break;
#endif
	case JTAGICE_CMND_GO:
		_RSP_OK();
		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_SINGLE_STEP:
		_RSP_OK();
		break;
	case JTAGICE_CMND_FORCED_STOP:
		AVR_JTAGOCD_ForceStop();
		_RSP_OK();
		break;
#endif
	case JTAGICE_CMND_RESET:
		AVR_JTAG_SendIns(AVR_JTAG_INS_AVR_RESET);
		JTAG_TAP_Data_1Byte(1,1,JTAG_TAP_ENTER_IDLE);
		DelayMS(1);
		AVR_JTAG_SendIns(AVR_JTAG_INS_AVR_RESET);
		JTAG_TAP_Data_1Byte(0,1,JTAG_TAP_ENTER_IDLE);

//DelayMS(200);
#if 0
// For Test
AVR_JTAGOCD_CheckPC(tdo);

dat[9] = tdo[0];
dat[10] = tdo[1];
dat[11] = tdo[2];
dat[12] = tdo[3];
dat[13] = tdi[0];
dat[14] = tdi[1];
dat[15] = tdi[2];
dat[16] = tdi[3];
_RSP_MEMORY(8);
break;
#endif
#if AVRD_JTAG_SHOULD_ENABLE
		Target_MCU_State = 1;		// Running
#endif
		_RSP_OK();
		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_ERASEPAGE_SPM:
		break;
#endif
#endif
#if AVRP_JTAG_SHOULD_ENABLE
//	case JTAGICE_CMND_SELFTEST:
//		for(tmp8 = 0;tmp8 < 8;tmp8++)
//			dat[9 + tmp8] = 1;
//		_RSP_SELFTEST();
//		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_SET_BREAK:
		break;
	case JTAGICE_CMND_GET_BREAK:
		break;
#endif
	case JTAGICE_CMND_CHIP_ERASE:
		AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
		AVR_JTAG_PROG_ChipErase();
		while(!AVR_JTAG_PROG_ChipEraseComplete());

		_RSP_OK();
		break;
	case JTAGICE_CMND_ENTER_PROGMODE:
		if(!AVRP_Programming)
		{
#if ISP_JTAG_AutoPower_En
#if !ISP_JTAG_FourcePower_En
			if(Vtarget < Vtarget_HasPower)
#endif
			{
				ISP_JTAG_PowerOn();
				DelayMS(5);
			}
#endif
			AVRP_Programming = 2;
			JTAG_TAP_Init();
			JTAG_TAP_Reset();
		}

		AVR_JTAG_SendIns(AVR_JTAG_INS_AVR_RESET);
		AVR_JTAG_SendDat(1,AVR_JTAG_REG_Reset_Len);

		AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_ENABLE);
		AVR_JTAG_SendDat(0xA370,AVR_JTAG_REG_ProgrammingEnable_Len);

DelayMS(10);

#if AVRD_JTAG_SHOULD_ENABLE || AVRD_DW_SHOULD_ENABLE
		Target_MCU_State = 2;	// Programming
#endif
		_RSP_OK();
		break;
	case JTAGICE_CMND_LEAVE_PROGMODE:
		AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_COMMANDS);
		AVR_JTAG_PROG_LoadNoOperationCommand();

		AVR_JTAG_SendIns(AVR_JTAG_INS_PROG_ENABLE);
		AVR_JTAG_SendDat(0,AVR_JTAG_REG_ProgrammingEnable_Len);

		AVR_JTAG_SendIns(AVR_JTAG_INS_AVR_RESET);
		AVR_JTAG_SendDat(0,AVR_JTAG_REG_Reset_Len);

		JTAG_TAP_Fini();
		AVRP_Programming = 0;	
	
#if ISP_JTAG_AutoPower_En
		ISP_JTAG_PowerOff();
#endif
		_RSP_OK();
		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_SET_N_PARAMETERS:
		break;
	case JTAGICE_CMND_CLR_BREAK:
		break;
	case JTAGICE_CMND_RUN_TO_ADDR:
		break;
#endif
//	case JTAGICE_CMND_SPI_CMD:
//		break;
#if AVRD_JTAG_SHOULD_ENABLE
	case JTAGICE_CMND_CLEAR_EVENTS:
		break;
	case JTAGICE_CMND_RESTORE_TARGET:
		JTAG_TAP_Fini();
#if ISP_JTAG_AutoPower_En
		ISP_JTAG_PowerOff();
#endif
		AVRP_Programming = 0;	

		_RSP_OK();
		break;
#endif
#endif
	default:
		_RSP_FAILED();
		break;
	}

	return 0;
}
#endif		// #if AVRP_JTAG_SHOULD_ENABLE
