////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	AVR_Device_Phenix.h
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	headerfile for Offline programming
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////

#define AVRP_Phenix_Debug								0
#define AVRP_Phenix_Debug_Out(data,len)					

#define AVRP_Phenix_BlockInStaticFlash					1
#define AVRP_Phenix_BlockInDynamicFlash					2
#define AVRP_Phenix_BlockInDataFlash					3

#define AVRP_Phenix_BlockPosition						AVRP_Phenix_BlockInStaticFlash

/// Data Area Type(System_Information,Interface_Information and Target_Information)
#define AVRP_Phenix_DataAddrType						uint16

#if AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInStaticFlash
#define AVRP_Phenix_DataAddr							AVRP_Phenix_BlockData
#define AVRP_Phenix_DataSize							sizeof(AVRP_Phenix_DebugData)
#elif AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInDynamicFlash
#define AVRP_Phenix_DataAddr							(16 * 1024)
#define AVRP_Phenix_DataSize							(16 * 1024)
#elif AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInDataFlash
#define AVRP_Phenix_DataAddr							0
#define AVRP_Phenix_DataSize							(16 * 1024)
#endif

// Warning: addr can ONLY be used once
#if (AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInStaticFlash) || (AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInDynamicFlash)
/// read one byte from addr
#define AVRP_Phenix_GetDataByte(addr)					(uint8)pgm_read_byte(AVRP_Phenix_DataAddr + (addr))
/// read one word from addr
#define AVRP_Phenix_GetDataWord(addr)					(uint16)pgm_read_word(AVRP_Phenix_DataAddr + (addr))
/// copy len(in byte) data from addrData(in Flash) to addrRAM(in RAM)
#define AVRP_Phenix_CopyData(addrRAM,addrData,len)		memcpy_P((void*)(addrRAM),(PGM_VOID_P)(AVRP_Phenix_DataAddr + (addrData)),(size_t)(len))
/// Compare len(in byte) between addrData(in Flash) and addrRAM(in RAM)
#define AVRP_Phenix_CompareData(addrRAM,addrData,len)	memcmp_P((const void*)(addrRAM),(PGM_VOID_P)(AVRP_Phenix_DataAddr + (addrData)),(size_t)(len))
/// read one address from addr
#define AVRP_Phenix_GetAddr(addr)						AVRP_Phenix_GetDataWord(addr)
#elif (AVRP_Phenix_BlockPosition == AVRP_Phenix_BlockInDataFlash)
#define AVRP_Phenix_GetDataByte(addr)					
#define AVRP_Phenix_GetDataWord(addr)					
#define AVRP_Phenix_CopyData(addrRAM,addrData,len)		
#define AVRP_Phenix_CompareData(addrRAM,addrData,len)	
#define AVRP_Phenix_GetAddr(addr)						
#endif

// Block(Interface_Information or Target_Information)
#define AVRP_Phenix_PackageInfoSize						sizeof(PackageInfo)
/// read PackageInfo Struct from Package Information Array
#define AVRP_Phenix_GetPackageInfo(block,base,offset)	AVRP_Phenix_CopyData((block),(base) + AVRP_Phenix_InterfaceAddr_PackageArrayAttr + (offset) * AVRP_Phenix_PackageInfoSize,AVRP_Phenix_PackageInfoSize)
#define AVRP_Phenix_TargetAddrSize						sizeof(AVRP_Phenix_DataAddrType)
/// read Target Address from Target Address Information Array
#define AVRP_Phenix_GetTargetAddr(base,offset)			((base) + AVRP_Phenix_TargetAddr_ProgramData + AVRP_Phenix_GetDataWord((base) + AVRP_Phenix_TargetAddr_ProgramDataArray + (offset) * AVRP_Phenix_TargetAddrSize))
// Constants
#define AVRP_Phenix_Constants_NumOfPackages				14
#define AVRP_Phenix_Constants_NumOfInterfaces			AVRP_Phenix_Constraints_MaxNumOfInterfaces
#define AVRP_Phenix_Constants_NumOfTargets				AVRP_Phenix_Constraints_MaxNumOfTargets

// Constraints
#define AVRP_Phenix_Constraints_MaxNumOfInterfaces		8
#define AVRP_Phenix_Constraints_MaxNumOfTargets			8

// System Information
#define AVRP_Phenix_SysInfo_AddrArraySize				4
#define AVRP_Phenix_SysAddr_Valid						0
#define AVRP_Phenix_SysAddr_NumOfInterfaces				1
#define AVRP_Phenix_SysAddr_NumOfTargets				2
#define AVRP_Phenix_SysAddr_InterfaceDataArray			3
#define AVRP_Phenix_SysAddr_TargetDataArray				(3 + AVRP_Phenix_Constants_NumOfInterfaces * 4) 

// Interface Information
#define AVRP_Phenix_InterfaceAddr_ProgramMode			0
#define AVRP_Phenix_InterfaceAddr_SizeOfEEPage			1
#define AVRP_Phenix_InterfaceAddr_SizeOfFlashPage		2
#define AVRP_Phenix_InterfaceAddr_PackageArrayAttr		16
#define AVRP_Phenix_InterfaceAddr_PackageAddr			44

// Package Information
#define AVRP_Phenix_PackageAddr_AddrIdx					0
#define AVRP_Phenix_PackageAddr_AddrLen					1
#define AVRP_Phenix_PackageAddr_DataIdx					2
#define AVRP_Phenix_PackageAddr_DataLen					3
#define AVRP_Phenix_PackageAddr_PackageStart			5

// Target Information
#define AVRP_Phenix_TargetAddr_Interfaces				0
#define AVRP_Phenix_TargetAddr_Operations				1
#define AVRP_Phenix_TargetAddr_Speed					3
#define AVRP_Phenix_TargetAddr_ProgramDataArray			4
#define AVRP_Phenix_TargetAddr_ProgramData				36

// Program Data Information
#define AVRP_Phenix_ProgramDataArray_Size				4
#define AVRP_Phenix_ProgramDataArrayIdx_Signature		0
#define AVRP_Phenix_ProgramDataArrayIdx_FuseBefore		1
#define AVRP_Phenix_ProgramDataArrayIdx_EraseCmd		2
#define AVRP_Phenix_ProgramDataArrayIdx_FuseAfter		3
#define AVRP_Phenix_ProgramDataArrayIdx_Calibrate		4
#define AVRP_Phenix_ProgramDataArrayIdx_OSCCAL			5
#define AVRP_Phenix_ProgramDataArrayIdx_Special			6
#define AVRP_Phenix_ProgramDataArrayIdx_Lockbits		7
#define AVRP_Phenix_ProgramDataArrayIdx_EEPROM			14
#define AVRP_Phenix_ProgramDataArrayIdx_Flash			15

// Operations
#define AVRP_Phenix_Operation_CheckSignature			0
#define AVRP_Phenix_Operation_ProgramFuseBefore			1
#define AVRP_Phenix_Operation_EraseChip					2
#define AVRP_Phenix_Operation_ProgramFlash				3
#define AVRP_Phenix_Operation_VerifyFlash				4
#define AVRP_Phenix_Operation_ProgramEEPROM				5
#define AVRP_Phenix_Operation_VerifyEEPROM				6
#define AVRP_Phenix_Operation_ProgramFuseAfter			7
#define AVRP_Phenix_Operation_VerifyFuse				8
#define AVRP_Phenix_Operation_CalibrateOSC				9
#define AVRP_Phenix_Operation_WriteOSCCAL				10
#define AVRP_Phenix_Operation_DoSpecial0				11
#define AVRP_Phenix_Operation_DoSpecial1				12
#define AVRP_Phenix_Operation_DoSpecial2				13
#define AVRP_Phenix_Operation_ProgramLockbits			14
#define AVRP_Phenix_Operation_VerifyLockbits			15

// Operations Enable
#define AVRP_Phenix_OperationEn_CheckSignature			1
#define AVRP_Phenix_OperationEn_ProgramFuseBefore		0
#define AVRP_Phenix_OperationEn_EraseChip				1
#define AVRP_Phenix_OperationEn_ProgramFlash			1
#define AVRP_Phenix_OperationEn_VerifyFlash				1
#define AVRP_Phenix_OperationEn_ProgramEEPROM			0
#define AVRP_Phenix_OperationEn_VerifyEEPROM			0
#define AVRP_Phenix_OperationEn_ProgramFuseAfter		1
#define AVRP_Phenix_OperationEn_VerifyFuse				1
#define AVRP_Phenix_OperationEn_CalibrateOSC			0
#define AVRP_Phenix_OperationEn_WriteOSCCAL				0
#define AVRP_Phenix_OperationEn_DoSpecial0				0
#define AVRP_Phenix_OperationEn_DoSpecial1				0
#define AVRP_Phenix_OperationEn_DoSpecial2				0
#define AVRP_Phenix_OperationEn_ProgramLockbits			1
#define AVRP_Phenix_OperationEn_VerifyLockbits			1

// Address type
/// Special Address Type Start
#define AVRP_Phenix_AddrType_SpecialStart				0xF0
/// Special Address Type used when Program Fuse thru ISP
#define AVRP_Phenix_AddrType_ISPWriteFuse				0xF0
/// Special Address Type used when Read Fuse thru ISP
#define AVRP_Phenix_AddrType_ISPReadFuse				0xF1

// Error
/// return Success
#define AVRP_Phenix_Error_Success						0
/// return Error: Parameter Error
#define AVRP_Phenix_Error_ParameterError				1

// Block Sizes
#define AVRP_Phenix_BlockSize_SysInfo					67
#define AVRP_Phenix_BlockSize_InterfaceISP				
#define AVRP_Phenix_BlockSize_InterfaceHVPP				
#define AVRP_Phenix_BlockSize_InterfaceHVSP				

void AVRP_Phenix_Main(void);
