////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	ISP.c
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	ISP Programming Protocol for AVR XMega
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <avr/io.h>

#include "../../type.h"
#include "../../app_cfg.h"

#include "../../MCU/gpio.h"

#include "SPI/spi.h"
#include "ISP.h"

#if AVRP_ISP_SHOULD_ENABLE

#if ISP_AutoSpeed_En
/// ISP Speed Array used to be set in AutoSpeed Mode
uint8 ISP_Speed[10] = {0,1,2,3,4,5,6,0x3E,0x95,0xA3};
#endif

/// ISP Wait Ready
/// @return		Success or not
uint8 ISP_RDY_Wait()
{
	uint8 dly = 0xFF,cmd[4] = {0xF0,0x00,0x00,0x00},ret[4];

poll:
	ISP_Comm((uint8*)cmd,(uint8*)ret);		// poll RDY/BSY
	if((ret[3] & 1) && (--dly > 0))
		goto poll;

	if(dly)
		return ISP_Success;
	else
		return ISP_Error;
}

/// ISP Communicate
/// @param[in]		data	data to send
/// @param[out]		ret		data received
void ISP_Comm(uint8 *data,uint8 *ret)
{
	uint8 tmp;

	if(SPI_Emu)
	{
		for(tmp = 0;tmp < 4;tmp++)
		{
			ret[tmp] = SPI_RW_Emu(data[tmp]);
		}
	}
	else
	{
		tmp = 0;
		SPI_SetData(data[tmp]);
		for(;tmp < 3;tmp++)
		{
			ret[tmp] = SPI_RW_HW(data[tmp + 1]);
		}
		SPI_WaitReady();
		ret[tmp] = SPI_GetData();
	}
}

/// ISP Initialization
/// @return
void ISP_Init(void)
{
	ISP_SetRSTOutput();
	SPI_Init();
}

/// ISP Finilization
/// @return
void ISP_Fini(void)
{
	SPI_Disable();
	SPI_SetSCKInput();
	SPI_SetMOSIInput();
	SPI_SetMISOInput();
	ISP_SetRSTInput();
}
#endif		// #if AVRP_ISP_SHOULD_ENABLE
