////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	AVR32_JTAG.h
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	include file for JTAG Programming for AVR32
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////

// Instructions:
#define AVR32_JTAG_INS_Len						5
#define AVR32_JTAG_INS_IDCODE					0x01
#define AVR32_JTAG_INS_SAMPLE_PRELOAD			0x02
#define AVR32_JTAG_INS_EXTEST					0x03
#define AVR32_JTAG_INS_INTEST					0x04
#define AVR32_JTAG_INS_CLAMP					0x06
#define AVR32_JTAG_INS_AVR_RESET				0x0C
#define AVR32_JTAG_INS_NEXUS_ACCESS				0x10
#define AVR32_JTAG_INS_MEMORY_WORD_ACCESS		0x11
#define AVR32_JTAG_INS_MEMORY_BLOCK_ACCESS		0x12
#define AVR32_JTAG_INS_CANCEL_ACCESS			0x13
#define AVR32_JTAG_INS_MEMORY_SERVICE			0x14
#define AVR32_JTAG_INS_MEMORY_SIZED_ACCESS		0x15
#define AVR32_JTAG_INS_SYNC						0x17
#define AVR32_JTAG_INS_HALT						0x1C
#define AVR32_JTAG_INS_BYPASS					0x1F

#define AVR32_NexusAccess()						
#define AVR32_MemoryService()					

#define AVR32_ReadMemoryWord(addr)				
#define AVR32_WriteMemoryWord(addr,dat)			
#define AVR32_ReadMemory(addr,size)				
#define AVR32_WriteMemory(addr,size,dat)		
#define AVR32_ReadBlock()						
#define AVR32_WriteBlock()						
#define AVR32_CancelAccess()					AVR32_JTAG_Operate8(AVR32_JTAG_INS_CANCEL_ACCESS,1,1)
#define AVR32_Sync(cnt)							AVR32_JTAG_Operate16(AVR32_JTAG_INS_SYNC,cnt,16)
#define AVR32_Reset(type)						AVR32_JTAG_Operate8(AVR32_JTAG_INS_AVR_RESET,type,5)
#define AVR32_Halt()							AVR32_JTAG_Operate8(AVR32_JTAG_INS_HALT,1,1)


#define AVR32_JTAG_SendIns(ins)					JTAG_TAP_Instr((uint8)(ins),AVR32_JTAG_INS_Len)
#define AVR32_JTAG_SendDat(datin,adtout,len)	JTAG_TAP_Data((datin),(datout),(len),JTAG_TAP_ENTER_IDLE)
#define AVR32_JTAG_Operate8(ins,datin,len)		(AVR32_JTAG_SendIns(ins),JTAG_TAP_Data_1Byte(datin,len,AVR32_JTAG_INS_Len))
#define AVR32_JTAG_Operate16(ins,datin,len)		(AVR32_JTAG_SendIns(ins),JTAG_TAP_Data_2Bytes(datin,len,AVR32_JTAG_INS_Len))
#define AVR32_JTAG_Operate(ins,datin,datout,len)	(AVR32_JTAG_SendIns(ins),AVR32_JTAG_SendDat(datin,len,AVR32_JTAG_INS_Len))
