////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	JTAG_TAP.h
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	include file for JTAG Test Access Port
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////
#define JTAG_TAP_ENTER_IDLE				1

#define JTAG_TAP_Idle()					(JTAG_TAP_TCK_CLR(),JTAG_TAP_TMS_SET(),JTAG_TAP_TDI_SET())

void JTAG_TAP_ChangeStatus(uint8 tms,uint8 len);

/// JTAG TAP Reset
#define JTAG_TAP_Reset()				JTAG_TAP_ChangeStatus(0x1F,6)
/// JTAG TAP Enter DR Mode
#define JTAG_TAP_Enter_DR_Mode()		JTAG_TAP_ChangeStatus(0x01,3)
/// JTAG TAP Enter IR Mode
#define JTAG_TAP_Enter_IR_Mode()		JTAG_TAP_ChangeStatus(0x03,4)
/// JTAG TAP Enter Idle Mode
#define JTAG_TAP_Enter_Idle_Mode()		JTAG_TAP_ChangeStatus(0x00,1)

extern uint8 JTAG_Clock;

uint8 JTAG_TAP_ShiftDataInOut_1Byte(uint8 tdi,uint8 len);
void JTAG_TAP_ShiftDataInOut(uint8 *tdi,uint8 *tdo,uint16 len);

void JTAG_TAP_Instr(uint8 instr,uint8 len);
uint8 JTAG_TAP_Data_1Byte(uint8 tdi,uint8 len,uint8 idle);
uint16 JTAG_TAP_Data_2Bytes(uint16 tdi,uint8 len,uint8 idle);
void JTAG_TAP_Data(uint8 *tdi,uint8 *tdo,uint16 len,uint8 idle);
#define JTAG_TAP_DataIn(tdo,len,idle)		JTAG_TAP_Data((uint8*)0,tdo,len,idle)
#define JTAG_TAP_DataOut(tdi,len,idle)		JTAG_TAP_Data(tdi,(uint8*)0,len,idle)
void JTAG_TAP_Init(void);
void JTAG_TAP_Fini(void);
